/***************************************************************************
 *            audio_oss.c
 *
 *  Fri Aug  6 15:17:58 2004
 *  Copyright  2004  benjie
 ****************************************************************************/

#include <gtk/gtk.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <linux/ioctl.h>
#include <sys/soundcard.h>

#include "support.h"
#include "tocdoc.h"
#include "timeline.h"
#include "audio_oss.h"

// OSS Specific Declarations
gint     oss_audio_fd;

// OSS Specific Audio Functions

gboolean oss_startAudio(void)
{
    gint    rate, format, channels;
    
    if ((oss_audio_fd = open("/dev/dsp", O_WRONLY | O_NONBLOCK, 0)) == -1) {
        perror("/dev/dsp");
        ShowErrorMessage("Sorry, I can't play sound right now.  "
                         "Maybe another app is already playing sound...");
        return FALSE;
    }
    
    format = AFMT_S16_LE;
    if(ioctl(oss_audio_fd, SNDCTL_DSP_SETFMT, &format) == -1) {
        perror("sound format");
        close(oss_audio_fd);
        return FALSE;
    }
    
    channels = 2;
    if(ioctl(oss_audio_fd, SNDCTL_DSP_CHANNELS, &channels) == -1) {
        perror("sound channels");
        close(oss_audio_fd);
        return FALSE;
    }
    
    rate = 44100;
    if(ioctl(oss_audio_fd, SNDCTL_DSP_SPEED, &rate) == -1) {
        perror("sound rate");
        close(oss_audio_fd);
        return FALSE;
    }
    return 1;
}


glong oss_writeBuffer(gchar *buffer, glong bytes)
{
    gint lenW, lenW2;
    
    if ((lenW = write(oss_audio_fd, buffer, bytes)) == -1) {
        perror("audio write");
        return -1;
    }
    
    if(lenW < bytes){
        printf("sleeping due to short write(%d/%d) mismatch.\n", lenW, bytes);
        ms_sleep(250);
        if((lenW2 = write(oss_audio_fd, buffer+lenW, bytes-lenW)) == -1) {
            perror("audio re-write");
            return -1;
        }
        return lenW+lenW2;
    }
    return lenW;
}


gboolean oss_syncAudio(void)
{
    ioctl(oss_audio_fd, SNDCTL_DSP_SYNC, NULL);
    return 1;
}


gboolean oss_resetAudio(void)
{
    ioctl(oss_audio_fd, SNDCTL_DSP_RESET, NULL);
    return 1;
}


void oss_closeAudio(void)
{
    close(oss_audio_fd);
}
