/***************************************************************************
 *            tocdoc.h
 *
 *  Fri Jul 23 11:44:32 2004
 *  Copyright  2004  benjie
 ****************************************************************************/

#ifndef _TOCDOC_H
#define _TOCDOC_H

#include <gtk/gtk.h>
#include <stdio.h>

typedef enum {teBadSound=-2, teNotSound=-1, teSilence=0, teAudio=1} teType;

typedef struct {
    unsigned char   *data;
    glong           length;
    gint            status;
    gint            useCount;
} thumbType;

typedef struct {
    GList       *item;       // link to the GList item in the Toc
	gchar       *filename;
    glong       fileLength_F;// length of file in frames
    teType      fileType;
	glong       start_F;     // redundant?
	glong       offset_F;    // how many frames into the file we start the track
	glong       pregapLength_F;    // how many frames of silence precede the track
	glong       audioLength_F;    // how many frames of the file we play in this track
	GList       *breaks;
    thumbType   *thumbnail;
    gboolean    selected;
} tocEntry;

typedef struct {
	GList       *entries;
	GList       *selection;
    gchar       *filename;
    gboolean    dirty;
} tocDoc;

typedef struct {
    gboolean    autoPregap;
    gint        maxCdLength;
    gchar       *cdrdaoCmdStr;
    gchar       *deviceStr;
    gchar       *driverStr;
    gboolean    eject;
} prefsStruct;

typedef gchar *(*renamingFunc)(gchar *filename);

tocDoc *newTocDoc(void);
tocDoc *openTocDoc(gchar *file);
tocDoc *cloneTocDoc(tocDoc *doc);
void freeTocDoc(tocDoc *doc);

gboolean readTocFromFile(tocDoc *doc, gchar *file);
void writeTocToFile(FILE *tocFile, tocDoc *doc, renamingFunc renamer);

glong tocDocLength(tocDoc *doc);
glong tocEntryLength(tocEntry *entry);
tocEntry *tocDocWhatIsAtTime(tocDoc *doc, glong t);

tocEntry *tocDocNewEntry(tocDoc *doc, gchar *filename, glong offset, glong pregap, glong length, gboolean addBreak);
void tocEntrySetFilename(tocEntry *newEntry, gchar *filename);
tocEntry *tocDocInsertEntryBefore(tocDoc *doc, tocEntry *newEntry, tocEntry *entry);
void tocDocMoveSelectionBefore(tocDoc *doc, tocEntry *entry);
tocEntry *tocDocAppendEntry(tocDoc *doc, tocEntry *newEntry);
void tocDocRemoveEntry(tocDoc *doc, tocEntry *entry);
void tocDocDeleteEntry(tocDoc *doc, tocEntry *entry);
void tocDocResizeEntry(tocDoc *doc, tocEntry *entry, glong length);
void tocDocResizePregap(tocDoc *doc, tocEntry *entry, glong pregap);
void tocDocSplitEntry(tocDoc *doc, tocEntry *entry, glong split);
gint tocEntryCountBreaks(tocEntry *entry);

void tocDocAddBreak(tocDoc *doc, glong time);
void tocDocRemoveBreak(tocDoc *doc, glong time);
glong tocDocFindClosestBreak(tocDoc *doc, glong time);

tocEntry *tocEntryNext(tocEntry *entry);
tocEntry *tocEntryPrevious(tocEntry *entry);

void tocSelect(tocDoc *doc, tocEntry *entry);
void tocDeselect(tocDoc *doc, tocEntry *entry);
void tocClearSelection(tocDoc *doc);
void tocDocDeleteSelection(tocDoc *doc);

void tocDocSetDirty(tocDoc *doc, gboolean dirty);

void openPrefs(void);
void savePrefs(void);

#endif
